@@ -11,6 +11,7 @@ tests_init \
 	libs_cflags_version_alt \
 	libs_cflags_version_different \
 	libs_cflags_version_different_bad \
+	libs_env \
 	exists_nonexitent \
 	nonexitent \
 	exists_version \
@@ -21,6 +22,8 @@ tests_init \
 	exists2 \
 	exists3 \
 	exists_version_alt \
+	exists_cflags \
+	exists_cflags_env \
 	uninstalled_bad \
 	uninstalled \
 	libs_intermediary \
@@ -29,13 +32,20 @@ tests_init \
 	libs_circular_directpc \
 	libs_static \
 	libs_static_ordering \
+	libs_metapackage \
+	license_isc \
+	license_noassertion \
+	modversion_noflatten \
 	pkg_config_path \
 	nolibs \
 	nocflags \
 	arbitary_path \
 	with_path \
 	relocatable \
-	single_depth_selectors
+	single_depth_selectors \
+	print_variables_env \
+	variable_env \
+	variable_no_recurse
 
 noargs_body()
 {
@@ -47,7 +57,7 @@ libs_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-L/test/lib -lfoo \n" \
+		-o inline:"-L/test/lib -lfoo\n" \
 		pkgconf --libs foo
 }
 
@@ -55,7 +65,7 @@ libs_cflags_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo\n" \
 		pkgconf --cflags --libs foo
 }
 
@@ -64,7 +74,7 @@ libs_cflags_version_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo\n" \
 		pkgconf --cflags --libs 'foo > 1.2'
 }
 
@@ -72,7 +82,7 @@ libs_cflags_version_multiple_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-I/test/include/foo -fPIC -L/test/lib -lbar -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lbar -lfoo\n" \
 		pkgconf --cflags --libs 'foo > 1.2 bar >= 1.3'
 }
 
@@ -80,7 +90,7 @@ libs_cflags_version_multiple_coma_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-I/test/include/foo -fPIC -L/test/lib -lbar -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lbar -lfoo\n" \
 		pkgconf --cflags --libs 'foo > 1.2,bar >= 1.3'
 }
 
@@ -88,7 +98,7 @@ libs_cflags_version_alt_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo\n" \
 		pkgconf --cflags --libs 'foo' '>' '1.2'
 }
 
@@ -96,7 +106,7 @@ libs_cflags_version_different_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo \n" \
+		-o inline:"-fPIC -I/test/include/foo -L/test/lib -lfoo\n" \
 		pkgconf --cflags --libs 'foo' '!=' '1.3.0'
 }
 
@@ -204,30 +214,30 @@ libs_intermediary_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-lintermediary-1 -lintermediary-2 -lfoo -lbar -lbaz \n" \
+		-o inline:"-lintermediary-1 -lintermediary-2 -lfoo -lbar -lbaz\n" \
 		pkgconf --libs intermediary-1 intermediary-2
 }
 
-libs_circular1_body()
-{
-	export PKG_CONFIG_PATH="${selfdir}/lib1"
-	atf_check \
-		-o inline:"-lcircular-1 -lcircular-2 -lcircular-3 \n" \
-		pkgconf --libs circular-1
-}
-
 libs_circular2_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-lcircular-3 -lcircular-1 -lcircular-2 \n" \
-		pkgconf --libs circular-3
+		-o inline:"circular-1: breaking circular reference (circular-1 -> circular-2 -> circular-1)\n" \
+		pkgconf circular-2 --validate
+}
+
+libs_circular1_body()
+{
+	export PKG_CONFIG_PATH="${selfdir}/lib1"
+	atf_check \
+		-o inline:"circular-3: breaking circular reference (circular-3 -> circular-1 -> circular-3)\n" \
+		pkgconf circular-1 --validate
 }
 
 libs_circular_directpc_body()
 {
 	atf_check \
-		-o inline:"-lcircular-1 -lcircular-2 -lcircular-3 \n" \
+		-o inline:"-lcircular-3 -lcircular-1 -lcircular-2\n" \
 		pkgconf --libs ${selfdir}/lib1/circular-3.pc
 }
 
@@ -235,7 +245,7 @@ libs_static_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"/libfoo.a -pthread \n" \
+		-o inline:"/libfoo.a -pthread\n" \
 		pkgconf --libs static-archive-libs
 }
 
@@ -243,28 +253,36 @@ libs_static_ordering_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1"
 	atf_check \
-		-o inline:"-L/test/lib -lbar -lfoo \n" \
+		-o inline:"-L/test/lib -lbar -lfoo\n" \
 		pkgconf --libs foo bar
 }
 
+libs_metapackage_body()
+{
+	export PKG_CONFIG_PATH="${selfdir}/lib1"
+	atf_check \
+		-o inline:"-L/test/lib -lbar -lfoo\n" \
+		pkgconf --static --libs metapackage-3
+}
+
 pkg_config_path_body()
 {
 	export PKG_CONFIG_PATH="${selfdir}/lib1${PATH_SEP}${selfdir}/lib2"
 	atf_check \
-		-o inline:"-L/test/lib -lfoo \n" \
+		-o inline:"-L/test/lib -lfoo\n" \
 		pkgconf --libs foo
 	atf_check \
-		-o inline:"-L/test/lib -lbar -lfoo \n" \
+		-o inline:"-L/test/lib -lbar -lfoo\n" \
 		pkgconf --libs bar
 }
 
 with_path_body()
 {
 	atf_check \
-		-o inline:"-L/test/lib -lfoo \n" \
+		-o inline:"-L/test/lib -lfoo\n" \
 		pkgconf --with-path=${selfdir}/lib1 --with-path=${selfdir}/lib2 --libs foo
 	atf_check \
-		-o inline:"-L/test/lib -lbar -lfoo \n" \
+		-o inline:"-L/test/lib -lbar -lfoo\n" \
 		pkgconf --with-path=${selfdir}/lib1 --with-path=${selfdir}/lib2 --libs bar
 }
 
@@ -288,7 +306,7 @@ arbitary_path_body()
 {
 	cp ${selfdir}/lib1/foo.pc .
 	atf_check \
-		-o inline:"-L/test/lib -lfoo \n" \
+		-o inline:"-L/test/lib -lfoo\n" \
 		pkgconf --libs foo.pc
 }
 
@@ -307,3 +325,67 @@ single_depth_selectors_body()
 		-o inline:"foo\n" \
 		pkgconf --with-path=${selfdir}/lib3 --print-requires bar
 }
+
+license_isc_body()
+{
+	atf_check \
+		-o inline:"foo: ISC\n" \
+		pkgconf --with-path=${selfdir}/lib1 --license foo
+}
+
+license_noassertion_body()
+{
+	atf_check \
+		-o inline:"bar: NOASSERTION\nfoo: ISC\n" \
+		pkgconf --with-path=${selfdir}/lib1 --license bar
+}
+
+modversion_noflatten_body()
+{
+	atf_check \
+		-o inline:"1.3\n" \
+		pkgconf --with-path=${selfdir}/lib1 --modversion bar
+}
+
+exists_cflags_body()
+{
+	atf_check \
+		-o inline:"-DHAVE_FOO\n" \
+		pkgconf --with-path=${selfdir}/lib1 --cflags --exists-cflags --fragment-filter=D foo
+}
+
+exists_cflags_env_body()
+{
+	atf_check \
+		-o inline:"FOO_CFLAGS='-DHAVE_FOO'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --cflags --exists-cflags --fragment-filter=D --env=FOO foo
+}
+
+libs_env_body()
+{
+	atf_check \
+		-o inline:"FOO_LIBS='-L/test/lib -lfoo'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --libs --env=FOO foo
+}
+
+print_variables_env_body()
+{
+	atf_check \
+		-o inline:"FOO_CFLAGS='-fPIC -I/test/include/foo'\nFOO_LIBS='-L/test/lib -lfoo'\nFOO_INCLUDEDIR='/test/include'\nFOO_LIBDIR='/test/lib'\nFOO_EXEC_PREFIX='/test'\nFOO_PREFIX='/test'\nFOO_PCFILEDIR='${selfdir}/lib1'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --env=FOO --print-variables --cflags --libs foo
+
+}
+
+variable_env_body()
+{
+	atf_check \
+		-o inline:"FOO_INCLUDEDIR='/test/include'\n" \
+		pkgconf --with-path=${selfdir}/lib1 --env=FOO --variable=includedir foo
+}
+
+variable_no_recurse_body()
+{
+	atf_check \
+		-o inline:"/test/include\n" \
+		pkgconf --with-path=${selfdir}/lib1 --variable=includedir bar
+}
